"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getArrayType = exports.getRelationProperty = exports.getPrimaryColumnType = exports.getPrimaryColumnName = exports.getPropsNullable = exports.getPropsType = exports.getProps = exports.getRelation = void 0;
const json_api_nestjs_1 = require("@klerick/json-api-nestjs");
const getRelation = (repository) => repository.metadata.relations.map((i) => {
    return i.propertyName;
});
exports.getRelation = getRelation;
const getProps = (repository) => {
    const relations = (0, exports.getRelation)(repository);
    return repository.metadata.columns
        .filter((i) => !relations.includes(i.propertyName))
        .map((r) => r.propertyName);
};
exports.getProps = getProps;
const getPropsType = (repository) => {
    const field = (0, exports.getProps)(repository);
    const entity = repository.target;
    const result = {};
    for (const item of field) {
        let typeProps = json_api_nestjs_1.TypeField.string;
        const fieldMetadata = repository.metadata.columns.find((i) => i.propertyName === item);
        if (fieldMetadata?.isArray) {
            result[item] = json_api_nestjs_1.TypeField.array;
            continue;
        }
        switch (Reflect.getMetadata('design:type', entity['prototype'], item)) {
            case Array:
                typeProps = json_api_nestjs_1.TypeField.array;
                break;
            case Date:
                typeProps = json_api_nestjs_1.TypeField.date;
                break;
            case Number:
                typeProps = json_api_nestjs_1.TypeField.number;
                break;
            case Boolean:
                typeProps = json_api_nestjs_1.TypeField.boolean;
                break;
            case Object:
                typeProps = json_api_nestjs_1.TypeField.object;
                break;
            default:
                typeProps = json_api_nestjs_1.TypeField.string;
        }
        result[item] = fieldMetadata?.isArray ? json_api_nestjs_1.TypeField.array : typeProps;
    }
    return result;
};
exports.getPropsType = getPropsType;
const getPropsNullable = (repository) => {
    const relation = (0, exports.getRelation)(repository);
    return repository.metadata.columns
        .filter((i) => !relation.includes(i.propertyName))
        .map((i) => i.isNullable || i.default !== undefined ? i.propertyName : false)
        .filter((i) => !!i);
};
exports.getPropsNullable = getPropsNullable;
const getPrimaryColumnName = (repository) => {
    const column = repository.metadata.primaryColumns.at(0);
    if (!column)
        throw new Error('Primary column not found');
    return column.propertyName;
};
exports.getPrimaryColumnName = getPrimaryColumnName;
const getPrimaryColumnType = (repository) => {
    const target = repository.target;
    const primaryColumn = repository.metadata.primaryColumns[0].propertyName;
    return (Reflect.getMetadata('design:type', target['prototype'], primaryColumn) ===
        Number
        ? json_api_nestjs_1.TypeField.number
        : json_api_nestjs_1.TypeField.string);
};
exports.getPrimaryColumnType = getPrimaryColumnType;
const getRelationProperty = (repository) => {
    return repository.metadata.relations.reduce((acum, item) => {
        // @ts-expect-error its dynamic creater
        acum[item.propertyName] = {
            entityClass: item.inverseEntityMetadata.target,
            nullable: item.isManyToMany || item.isOneToMany ? false : item.isNullable,
            isArray: item.isManyToMany || item.isOneToMany,
        };
        return acum;
    }, {});
};
exports.getRelationProperty = getRelationProperty;
const getArrayType = (repository) => {
    return Object.entries((0, exports.getPropsType)(repository))
        .filter(([name, type]) => type === json_api_nestjs_1.TypeField.array)
        .reduce((acum, [name]) => {
        const fieldMetadata = repository.metadata.columns.find((i) => i.propertyName === name);
        if (!fieldMetadata) {
            throw new Error('Metadata not found for field ' + name);
        }
        switch (fieldMetadata.type) {
            case 'number':
            case 'bigint':
            case 'smallint':
            case 'smalldecimal':
            case 'decimal':
            case 'double':
            case 'tinyint':
            case 'mediumint':
            case 'int':
            case 'float':
            case 'dec':
            case 'fixed':
            case 'numeric':
            case 'real':
            case 'int2':
            case 'int4':
            case 'int8':
            case 'integer':
                acum[name] = json_api_nestjs_1.TypeField.number;
                break;
            case 'character varying':
            case 'varying character':
            case 'char varying':
            case 'nvarchar':
            case 'national varchar':
            case 'character':
            case 'native character':
            case 'varchar':
            case 'char':
            case 'nchar':
            case 'national char':
            case 'varchar2':
            case 'nvarchar2':
            case 'alphanum':
            case 'shorttext':
            case 'raw':
            case 'binary':
            case 'varbinary':
            case 'string':
                acum[name] = json_api_nestjs_1.TypeField.string;
                break;
            case 'date':
            case 'timetz':
            case 'timestamptz':
            case 'timestamp with local time zone':
            case 'smalldatetime':
                acum[name] = json_api_nestjs_1.TypeField.date;
                break;
            case 'boolean':
                acum[name] = json_api_nestjs_1.TypeField.boolean;
        }
        return acum;
    }, {});
};
exports.getArrayType = getArrayType;
//# sourceMappingURL=index.js.map