import { OrmService, PatchData, PatchRelationshipData, PostData, PostRelationshipData, Query, QueryOne, EntityControllerParam, JsonApiTransformerService } from '@klerick/json-api-nestjs';
import { ResourceObject, RelationKeys, ResourceObjectRelationships } from '@klerick/json-api-nestjs-shared';
import { Repository } from 'typeorm';
import { TypeormUtilsService } from './typeorm-utils.service';
import { TypeOrmParam } from '../type';
export declare class TypeOrmService<E extends object, IdKey extends string = 'id'> implements OrmService<E, IdKey> {
    typeormUtilsService: TypeormUtilsService<E, IdKey>;
    transformDataService: JsonApiTransformerService<E, IdKey>;
    config: EntityControllerParam<TypeOrmParam>;
    repository: Repository<E>;
    getAll(query: Query<E, IdKey>): Promise<ResourceObject<E, 'array', null, IdKey>>;
    deleteOne(id: number | string): Promise<void>;
    deleteRelationship<Rel extends RelationKeys<E, IdKey>>(id: number | string, rel: Rel, input: PostRelationshipData): Promise<void>;
    getOne(id: number | string, query: QueryOne<E, IdKey>): Promise<ResourceObject<E, 'object', null, IdKey>>;
    getRelationship<Rel extends RelationKeys<E, IdKey>>(id: number | string, rel: Rel): Promise<ResourceObjectRelationships<E, IdKey, Rel>>;
    patchOne(id: number | string, inputData: PatchData<E, IdKey>): Promise<ResourceObject<E, 'object', null, IdKey>>;
    patchRelationship<Rel extends RelationKeys<E, IdKey>>(id: number | string, rel: Rel, input: PatchRelationshipData): Promise<ResourceObjectRelationships<E, IdKey, Rel>>;
    postOne(inputData: PostData<E, IdKey>): Promise<ResourceObject<E, 'object', null, IdKey>>;
    postRelationship<Rel extends RelationKeys<E, IdKey>>(id: number | string, rel: Rel, input: PostRelationshipData): Promise<ResourceObjectRelationships<E, IdKey, Rel>>;
}
