import { Repository } from 'typeorm';
import { RelationMetadata as TypeOrmRelationMetadata } from 'typeorm/metadata/RelationMetadata';
import { RelationKeys } from '@klerick/json-api-nestjs-shared';
import { PatchData, PostData, Query, UnionToTuple } from '@klerick/json-api-nestjs';
type RelationAlias<E> = {
    [K in UnionToTuple<RelationKeys<E>>[number] & PropertyKey]: string;
};
type RelationMetadata<E> = {
    [K in UnionToTuple<RelationKeys<E>>[number] & PropertyKey]: TypeOrmRelationMetadata;
};
type ResultQueryExpressionObject = {
    name: string;
    val: string;
};
type ResultQueryExpressionArray = {
    name: string;
    val: string;
}[];
export type RelationshipsResult<E extends object, IdKey extends string> = {
    [K in RelationKeys<E, IdKey>]: E[K] extends E[K][] ? E[K] : E[K] | null;
};
export type ResultQueryExpression = {
    alias?: string;
    expression: string;
    paramsForResult?: string[];
    params?: ResultQueryExpressionObject | ResultQueryExpressionArray;
    selectInclude?: string;
};
export type InputValidateData = {
    type: string;
    id: string;
};
export type ValidateReturn<T> = T extends unknown[] ? string[] : T extends null ? null : string;
export declare class TypeormUtilsService<E extends object, IdKey extends string = 'id'> {
    private repository;
    private readonly _currentAlias;
    private readonly _relationMetadata;
    private readonly _relationAlias;
    private readonly _relationFields;
    private readonly _entityMetadata;
    private _number;
    constructor(repository: Repository<E>);
    get currentAlias(): string;
    get relationFields(): UnionToTuple<RelationKeys<E>>;
    relationName(relName: keyof RelationAlias<E>): RelationAlias<E>[UnionToTuple<RelationKeys<E>, import("ts-toolbelt/out/Union/Last").Last<RelationKeys<E>>, [RelationKeys<E>] extends [never] ? true : false>[number] & PropertyKey];
    get currentPrimaryColumn(): keyof E;
    getAliasForRelation(relName: keyof RelationAlias<E>): string;
    getRelMetaDataForRelation(relName: keyof RelationAlias<E>): RelationMetadata<E>[UnionToTuple<RelationKeys<E>, import("ts-toolbelt/out/Union/Last").Last<RelationKeys<E>>, [RelationKeys<E>] extends [never] ? true : false>[number] & PropertyKey];
    getPrimaryColumnForRel(relName: keyof RelationAlias<E>): string;
    private getFilterObject;
    private get number();
    private getParamName;
    getAliasPath(fieldName: unknown): string;
    getAliasPath(fieldName: unknown, relname: keyof RelationAlias<E>, separator?: string): string;
    getAliasPath(fieldName: unknown, relname: string, separator?: string): string;
    private getSubQueryForManyToMany;
    getFilterExpressionForTarget(query: Query<E, IdKey>): ResultQueryExpression[];
    getFilterExpressionForRelation(query: Query<E, IdKey>): ResultQueryExpression[];
    private throwError;
    asyncIterateFindRelationships(relationships: NonNullable<PatchData<E, IdKey>['relationships'] | PostData<E, IdKey>['relationships']>): AsyncGenerator<RelationshipsResult<E, IdKey>>;
    saveEntityData(target: E, relationships: PatchData<E, IdKey>['relationships'] | PostData<E, IdKey>['relationships']): Promise<E>;
    validateRelationInputData<Rel extends RelationKeys<E, IdKey>, In extends InputValidateData | InputValidateData[]>(rel: Rel, inputData: In): Promise<ValidateReturn<In>>;
    validateRelationInputData<Rel extends RelationKeys<E, IdKey>, In extends InputValidateData | InputValidateData[]>(rel: Rel, inputData: In): Promise<ValidateReturn<In>>;
    validateRelationInputData<Rel extends RelationKeys<E, IdKey>, In extends null | InputValidateData | InputValidateData[]>(rel: Rel, inputData: In): Promise<ValidateReturn<In>>;
}
export {};
